package com.example.bluetoothgateway.Dialog;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.FileProvider;

import com.example.bluetoothgateway.R;

import java.io.File;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
//升级参考网址
//https://blog.csdn.net/u010356768/article/details/89212742
//https://www.cnblogs.com/IT-lss/p/10112909.html

public class DownloadDialog extends AlertDialog {
    private Context mContext;
    private LinearLayout progress_linlayout;
    private TextView mTextView;
    private ProgressBar mProgressBar;
    private RelativeLayout app_update_relayout;
    private TextView update_tv;
    private Button cancel_bt, confirm_bt;
    private View view;


    public DownloadDialog(Context context) {
        super(context);
        this.mContext = context;

    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //设置对话框样式
        setStyle();
        //初始化控件
        initView();
    }

    private void initView() {
        view = View.inflate(mContext, R.layout.download_dialog, null);
        progress_linlayout = (LinearLayout) view.findViewById(R.id.progress_linlayout);
        mTextView = (TextView) view.findViewById(R.id.mTextView);
        mProgressBar = (ProgressBar) view.findViewById(R.id.mProgressBar);
        app_update_relayout = (RelativeLayout) view.findViewById(R.id.app_update_relayout);
        update_tv = (TextView) view.findViewById(R.id.update_tv);
        cancel_bt = (Button) view.findViewById(R.id.cancel_bt);
        confirm_bt = (Button) view.findViewById(R.id.confirm_bt);
        setContentView(view);
        cancel_bt.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
        confirm_bt.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String filePath = Environment.getExternalStorageDirectory() + "/WDKLGuardian";
                File file = new File(filePath);
                File files[] = file.listFiles();
                String appName = files[0].getName();

                Intent intent = new Intent(Intent.ACTION_VIEW);
                File apkFile = new File(filePath+"/"+appName);
                // 由于没有在Activity环境下启动Activity,设置下面的标签
                intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                //Android 7.0以上要使用FileProvider
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                    intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                    //参数1 上下文, 参数2 Provider主机地址 和配置文件中保持一致   参数3  共享的文件
                    Uri uri = FileProvider.getUriForFile(mContext, mContext.getPackageName() + ".fileprovider", apkFile);
                    //添加这一句表示对目标应用临时授权该Uri所代表的文件
//                    intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                    intent.setDataAndType(uri, "application/vnd.android.package-archive");
                } else {
                    intent.setDataAndType(Uri.fromFile(apkFile), "application/vnd.android.package-archive");
                }
                mContext.startActivity(intent);
                dismiss();
            }
        });

    }

    private void setStyle() {
        //设置对话框不可取消
        this.setCancelable(false);
        //设置触摸对话框外面不可取消
        this.setCanceledOnTouchOutside(false);
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindow().getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        //获得应用窗口大小
        WindowManager.LayoutParams layoutParams = this.getWindow().getAttributes();
        //设置对话框居中显示
        layoutParams.gravity = Gravity.CENTER;
        //设置对话框宽度为屏幕的3/5
        layoutParams.width = (displaymetrics.widthPixels / 5) * 3;
    }

    //设置进度条
    public void setProgress(int progress) {
        mTextView.setText(progress + "%");
        mProgressBar.setProgress(progress);
    }

    public void handover(int tag) {
        if (tag == 0) {
            progress_linlayout.setVisibility(View.GONE);
            app_update_relayout.setVisibility(View.VISIBLE);
        } else {
            progress_linlayout.setVisibility(View.VISIBLE);
            app_update_relayout.setVisibility(View.GONE);
        }

    }
}
