package com.example.bluetoothgateway;

import android.Manifest;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;
import android.content.ComponentName;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.view.animation.LinearInterpolator;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.RequiresApi;

import com.clj.fastble.BleManager;
import com.clj.fastble.callback.BleGattCallback;
import com.clj.fastble.callback.BleNotifyCallback;
import com.clj.fastble.callback.BleScanCallback;
import com.clj.fastble.data.BleDevice;
import com.clj.fastble.exception.BleException;
import com.clj.fastble.scan.BleScanRuleConfig;
import com.clj.fastble.utils.HexUtil;
import com.example.bluetoothgateway.Dialog.DownloadDialog;
import com.example.bluetoothgateway.FastBle.adapter.DeviceAdapter;
import com.example.bluetoothgateway.FastBle.comm.Device;
import com.example.bluetoothgateway.FastBle.comm.ObserverManager;
import com.example.bluetoothgateway.base.BaseActivity;
import com.example.bluetoothgateway.net.BluetoothService;
import com.example.bluetoothgateway.net.OkHttp;
import com.example.bluetoothgateway.utils.BluetoothUtil;
import com.example.bluetoothgateway.utils.NetFunctionConfig;
import com.example.bluetoothgateway.utils.Utils;

import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import butterknife.Bind;
import butterknife.ButterKnife;

public class MainActivity extends BaseActivity implements View.OnClickListener, Utils.OnDomainNameTransferIPListener {

    private String TAG = MainActivity.class.getCanonicalName();
    @Bind(R.id.is_bluetooth_serve_tv)
    TextView is_bluetooth_serve_tv;

    @Bind(R.id.socket_serve_state_tv)
    TextView socket_serve_state_tv;
    @Bind(R.id.socket_is_on_line_tv)
    TextView socket_is_on_line_tv;
    @Bind(R.id.configuration_layout)
    LinearLayout configuration_layout;
    @Bind(R.id.setting_ip_edt)
    EditText setting_ip_edt;
    @Bind(R.id.setting_port_edt)
    EditText setting_port_edt;

    TextView setting_ip_tv, versionInformation_tv;
    Button cancel_bt, confirm_bt;

    private static final int REQUEST_CODE_OPEN_GPS = 1;
    private static final int REQUEST_CODE_PERMISSION_LOCATION = 2;
    //三诺的是这个固定值
    private static final String ServiceUuid = "0000ffb0-0000-1000-8000-00805f9b34fb";
    private static final String CharacteristicUuid = "0000ffb2-0000-1000-8000-00805f9b34fb";
    //艾科
    private static final String ServiceUuid2 = "11223344-5566-7788-99aa-bbccddeeff00";
    private static final String CharacteristicUuid2 = "00004a5b-0000-1000-8000-00805f9b34fb";

    private Intent BluetoothServiceIntent;
    private int count = 1;
    private BluetoothService.ServiceBinder serviceBinder;
    private Service service;

    private String ip = "";
    private String portStr = "";

    private OkHttp okHttp;
    private DownloadDialog downloadDialog;

    private Button btn_scan;
    private TextView send_data_tv;
    private ImageView img_loading;

    private Animation operatingAnim;
    private DeviceAdapter mDeviceAdapter;
    private ProgressDialog progressDialog;
    private Device singleDevice;

    private String datas = "";
    public static MainActivity mainActivity;

    public Timer timer;
    public TimerTask timerTask;

    private long backClick = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        ButterKnife.bind(this);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        mainActivity = this;
        init();

        Utils utils = new Utils(this, this);
        okHttp = new OkHttp();
        requestRunTimePermissions();
        //初始化蓝牙
        singleDevice = new Device();
        BleManager.getInstance().init(getApplication());
        BleManager.getInstance()
                .enableLog(true)
                .setReConnectCount(1, 5000)
                .setConnectOverTime(20000)
                .setOperateTimeout(5000);

        appCheckUpdate();


//        //蓝牙搜索
//        BlueToothFoundReceiver blueToothFoundReceiver = new BlueToothFoundReceiver();
//        IntentFilter filter_found = new IntentFilter(BluetoothDevice.ACTION_FOUND);
//        filter_found.addAction(BluetoothAdapter.ACTION_DISCOVERY_STARTED);
//        filter_found.addAction(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
//        registerReceiver(blueToothFoundReceiver, filter_found);


//        BluetoothServiceIntent = new Intent(this, BluetoothService.class);
//        startService(BluetoothServiceIntent);

//        //注册广播，蓝牙状态监听
//        BlueToothValueReceiver blueToothValueReceiver = new BlueToothValueReceiver();
//        IntentFilter filter = new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED);
//        registerReceiver(blueToothValueReceiver, filter);

//         new Handler().postDelayed(new Runnable() {
//             @Override
//             public void run() {
//                startActivity(new Intent(MainActivity.this,Main2Activity.class));
//             }
//         },3000);

    }

    /**
     * APP更新检测
     */
    private void appCheckUpdate() {
        //检测是否有更新
        if (okHttp == null) {
            okHttp = new OkHttp();
        }

        okHttp.APPUpdate(this, new OkHttp.OnRequestLitener() {
            @Override
            public void onStartTheDownload() {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
//                        downloadDialog = new DownloadDialog(MainActivity.this);
//                        downloadDialog.show();
//                        downloadDialog.handover(1);
                    }
                });
            }

            @Override
            public void onRequestSuccess(String data) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        if (downloadDialog != null) {
                            downloadDialog.handover(0);
                        } else {
                            downloadDialog = new DownloadDialog(MainActivity.this);
                            downloadDialog.show();
                            downloadDialog.handover(0);
                        }

                    }
                });

            }

            @Override
            public void onBeDownloading(final int progress) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
//                        downloadDialog.setProgress(progress);
                    }
                });
            }

            @Override
            public void onRequestFail(final String data) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        if (downloadDialog != null) {
                            downloadDialog.dismiss();
                        }
                        Toast.makeText(getApplication(), data, Toast.LENGTH_LONG).show();
                    }
                });

            }
        });

    }

    private void requestRunTimePermissions() {
        requestRunTimePermission(new OnCloseDialogListener() {
            @Override
            public void OnCloseDialog(AlertDialog alertDialog) {
                if (alertDialog != null) {
                    alertDialog.dismiss();
                }
            }
        });
    }

    private void init() {
        setting_ip_tv = findViewById(R.id.setting_ip_tv);
        cancel_bt = findViewById(R.id.cancel_bt);
        confirm_bt = findViewById(R.id.confirm_bt);
        btn_scan = (Button) findViewById(R.id.btn_scan);
        btn_scan.setText("开始扫描");
        send_data_tv = findViewById(R.id.send_data_tv);
        img_loading = (ImageView) findViewById(R.id.img_loading);
        versionInformation_tv = findViewById(R.id.versionInformation_tv);


        setting_ip_tv.setOnClickListener(this);
        cancel_bt.setOnClickListener(this);
        confirm_bt.setOnClickListener(this);
        btn_scan.setOnClickListener(this);
        send_data_tv.setOnClickListener(this);
        findViewById(R.id.tv_notice).setOnClickListener(this);
        versionInformation_tv.setText(Utils.getDevInformation(this));

        operatingAnim = AnimationUtils.loadAnimation(this, R.anim.rotate);
        operatingAnim.setInterpolator(new LinearInterpolator());
        progressDialog = new ProgressDialog(this);

        mDeviceAdapter = new DeviceAdapter(this);
        mDeviceAdapter.setOnDeviceClickListener(new DeviceAdapter.OnDeviceClickListener() {
            @Override
            public void onConnect(BleDevice bleDevice) {
                if (!BleManager.getInstance().isConnected(bleDevice)) {
                    BleManager.getInstance().cancelScan();
                    connect(bleDevice);
                }
            }

            @Override
            public void onDisConnect(final BleDevice bleDevice) {
                if (BleManager.getInstance().isConnected(bleDevice)) {
                    BleManager.getInstance().disconnect(bleDevice);
                }
            }

            @Override
            public void onDetail(BleDevice bleDevice) {
                if (BleManager.getInstance().isConnected(bleDevice)) {
//                    Intent intent = new Intent(Main2Activity.this, OperationActivity.class);
//                    intent.putExtra(OperationActivity.KEY_DATA, bleDevice);
//                    startActivity(intent);
                }
            }
        });
        ListView listView_device = (ListView) findViewById(R.id.list_device);
        listView_device.setAdapter(mDeviceAdapter);

    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.e(TAG, "onResume()..");

        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                if (BluetoothService.isServiceRunning()) {
                    is_bluetooth_serve_tv.setTextColor(Color.parseColor("#7CFC00"));
                    is_bluetooth_serve_tv.setText("已启动");
                } else {
                    is_bluetooth_serve_tv.setTextColor(Color.parseColor("#EE2C2C"));
                    is_bluetooth_serve_tv.setText("启动失败，请重新打开");
                }
                getConnectionStatus();
            }
        }, 2000);

    }

    /**
     * 获取连接状态
     */
    private void getConnectionStatus() {
        if (BluetoothService.connectState) {
            socket_is_on_line_tv.setTextColor(Color.parseColor("#7CFC00"));
            socket_is_on_line_tv.setText("连接");
        } else {
            socket_is_on_line_tv.setTextColor(Color.parseColor("#EE2C2C"));
            socket_is_on_line_tv.setText("未连接，请检查网络是否连接 或联系管理员");
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.socket_serve_state_tv:
                getConnectionStatus();
                break;
            case R.id.setting_ip_tv:
//                String data = "12FF10FFF3000001007e010000E06234C9F3D1";//三诺
//                String data = "12FF10FFF30075730071010000E06234CB17C7";
//                String data = "12FF10FFF3000001007b0100004ce173c4ccec";//艾科 两个一样

//                String data = "FF10FFF3000001"+"0011"+"000000"+"E06234C9F3D1";
                //我手
//                String data = "2644312031200632373838340D";
                //发数据
//                serviceBinder.setdata(data);

                /*if (count > 5) {
                    count = 0;
                    configuration_layout.setVisibility(View.VISIBLE);
                    setting_ip_edt.setText(NetFunctionConfig.getIp());
                    setting_port_edt.setText(NetFunctionConfig.getPort() + "");
                } else {
                    count++;
                }*/
                break;
            case R.id.cancel_bt:
                configuration_layout.setVisibility(View.GONE);
                break;
            case R.id.confirm_bt:
                ip = setting_ip_edt.getText().toString();
                portStr = setting_port_edt.getText().toString();
//                Pattern pattern = Pattern.compile("(25[0-5]|2[0-4]\\d|[0-1]\\d{2}|[1-9]?\\d)\\.(25[0-5]|2[0-4]\\d|[0-1]\\d{2}|[1-9]?\\d)\\.(25[0-5]|2[0-4]\\d|[0-1]\\d{2}|[1-9]?\\d)\\.(25[0-5]|2[0-4]\\d|[0-1]\\d{2}|[1-9]?\\d)");
//                Matcher matcher = pattern.matcher(ip);
//                boolean rs = matcher.matches();
//                if (!rs) {
//                    Toast.makeText(this, "设置失败，ip地址输入有误", Toast.LENGTH_LONG).show();
//                    return;
//                }
                boolean rs = Utils.isURL2(ip);
                if (!rs) {
                    Toast.makeText(this, "设置失败，域名或ip地址输入有误", Toast.LENGTH_LONG).show();
                    return;
                }
                Utils.parseHostGetIPAddress(ip);

                break;
            case R.id.btn_scan:
                if (btn_scan.getText().equals(getString(R.string.start_scan))) {
//                    setScanRule();
//                    startScan();
                    loopConnection();
                } else if (btn_scan.getText().equals(getString(R.string.stop_scan))) {
                    BleManager.getInstance().cancelScan();
                }
                break;
            case R.id.tv_notice:
                Log.d(TAG, "count=" + count);
                if (count > 9) {
                    count = 1;
                    //修改端口
                    int port = NetFunctionConfig.getPort();
                    if (port == 3006) {
                        NetFunctionConfig.setPort(3008);
                        Toast.makeText(this, "3008端口切换成功，请重启APP", Toast.LENGTH_LONG).show();
                    } else {
                        NetFunctionConfig.setPort(3006);
                        Toast.makeText(this, "3006端口切换成功，请重启APP", Toast.LENGTH_LONG).show();
                    }
                } else {
                    count++;
                }
                break;
        }
    }

    private void loopConnection(){
        if (timer != null) timer.purge();
        if (timerTask != null) timerTask.cancel();
        timer = new Timer();
        timerTask = new TimerTask() {
            @Override
            public void run() {

                Log.e(TAG,"重新扫描。。。");
                if (btn_scan.getText().equals(getString(R.string.stop_scan))) {
                    BleManager.getInstance().cancelScan();
                }
                setScanRule();
                startScan();
            }
        };
        timer.schedule(timerTask, 0, 10000);
    }


    /**
     * 初始化配置数据
     */
    private void setScanRule() {
//        String[] uuids;
//        String str_uuid = et_uuid.getText().toString();
//        if (TextUtils.isEmpty(str_uuid)) {
//            uuids = null;
//        } else {
//            uuids = str_uuid.split(",");
//        }
//        UUID[] serviceUuids = null;
//        if (uuids != null && uuids.length > 0) {
//            serviceUuids = new UUID[uuids.length];
//            for (int i = 0; i < uuids.length; i++) {
//                String name = uuids[i];
//                String[] components = name.split("-");
//                if (components.length != 5) {
//                    serviceUuids[i] = null;
//                } else {
//                    serviceUuids[i] = UUID.fromString(uuids[i]);
//                }
//            }
//        }
//
//        String[] names;
//        String str_name = et_name.getText().toString();
//        if (TextUtils.isEmpty(str_name)) {
//            names = null;
//        } else {
//            names = str_name.split(",");
//        }
//
//        String mac = et_mac.getText().toString();
//
//        boolean isAutoConnect = sw_auto.isChecked();


        String[] names = {"BDE_WEIXIN_TTM", "OGM_112404X200E2C3"};
        BleScanRuleConfig scanRuleConfig = new BleScanRuleConfig.Builder()
//                .setServiceUuids(serviceUuids)      // 只扫描指定的服务的设备，可选
                .setDeviceName(true, names)   // 只扫描指定广播名的设备，可选
//                .setDeviceMac(mac)                  // 只扫描指定mac的设备，可选
                .setAutoConnect(true)      // 连接时的autoConnect参数，可选，默认false
                .setScanTimeOut(10000)              // 扫描超时时间，可选，默认10秒
                .build();
        BleManager.getInstance().initScanRule(scanRuleConfig);
    }

    private void startScan() {
        BleManager.getInstance().scan(new BleScanCallback() {
            @Override
            public void onScanStarted(boolean success) {
                mDeviceAdapter.clearScanDevice();
                mDeviceAdapter.notifyDataSetChanged();
                img_loading.startAnimation(operatingAnim);
                img_loading.setVisibility(View.VISIBLE);
                btn_scan.setText(getString(R.string.stop_scan));
            }

            @Override
            public void onLeScan(BleDevice bleDevice) {
                super.onLeScan(bleDevice);
            }

            @Override
            public void onScanning(BleDevice bleDevice) {
                mDeviceAdapter.addDevice(bleDevice);
                mDeviceAdapter.notifyDataSetChanged();
                if (bleDevice.getName().equals("BDE_WEIXIN_TTM") || bleDevice.getName().equals("OGM_112404X200E2C3")) {
                    Log.e(TAG, "找到需要的设备");
                    if (!BleManager.getInstance().isConnected(bleDevice)) {
                        BleManager.getInstance().cancelScan();
                        connect(bleDevice);
                    }
                }
            }

            @Override
            public void onScanFinished(List<BleDevice> scanResultList) {
                img_loading.clearAnimation();
                img_loading.setVisibility(View.INVISIBLE);
                btn_scan.setText(getString(R.string.start_scan));
            }
        });
    }


    /**
     * 连接蓝牙
     *
     * @param bleDevice
     */
    private void connect(final BleDevice bleDevice) {
        BleManager.getInstance().connect(bleDevice, new BleGattCallback() {
            @Override
            public void onStartConnect() {
                progressDialog.show();
            }

            @Override
            public void onConnectFail(BleDevice bleDevice, BleException exception) {
                img_loading.clearAnimation();
                img_loading.setVisibility(View.INVISIBLE);
                btn_scan.setText(getString(R.string.start_scan));
                progressDialog.dismiss();
                Toast.makeText(MainActivity.this, getString(R.string.connect_fail), Toast.LENGTH_LONG).show();
            }

            @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
            @Override
            public void onConnectSuccess(final BleDevice bleDevice, BluetoothGatt gatt, int status) {
                progressDialog.dismiss();
                mDeviceAdapter.addDevice(bleDevice);
                mDeviceAdapter.notifyDataSetChanged();

                for (BluetoothGattService service : gatt.getServices()) {
                    //三诺的 和艾科的
                    if (service.getUuid().toString().equals(ServiceUuid) || service.getUuid().toString().equals(ServiceUuid2)) {
                        for (final BluetoothGattCharacteristic characteristic : service.getCharacteristics()) {
                            if (characteristic.getUuid().toString().equals(CharacteristicUuid) ||
                                    characteristic.getUuid().toString().equals(CharacteristicUuid2)) {
                                int charaProp = characteristic.getProperties();
                                if ((charaProp & BluetoothGattCharacteristic.PROPERTY_NOTIFY) > 0) {
                                    singleDevice.setDevice(bleDevice);
                                    singleDevice.setCharacteristic(characteristic);
                                    //三诺
                                    if ((charaProp & BluetoothGattCharacteristic.PROPERTY_WRITE_NO_RESPONSE) > 0) {
                                        singleDevice.setWriteType(BluetoothGattCharacteristic.PROPERTY_WRITE_NO_RESPONSE);
                                    } else if ((charaProp & BluetoothGattCharacteristic.PROPERTY_WRITE) > 0) {//艾科
                                        singleDevice.setWriteType(BluetoothGattCharacteristic.PROPERTY_WRITE);
                                    }

                                    //打开通知
                                    BleManager.getInstance().notify(
                                            bleDevice,
                                            characteristic.getService().getUuid().toString(),
                                            characteristic.getUuid().toString(),
                                            new BleNotifyCallback() {

                                                @Override
                                                public void onNotifySuccess() {
                                                    runOnUiThread(new Runnable() {
                                                        @Override
                                                        public void run() {
                                                            // 打开通知操作成功
                                                            Log.i(TAG, "打开通知成功");
                                                            Log.e(TAG, "mac:" + bleDevice.getMac());
                                                        }
                                                    });
                                                }

                                                @Override
                                                public void onNotifyFailure(final BleException exception) {
                                                    runOnUiThread(new Runnable() {
                                                        @Override
                                                        public void run() {
                                                            // 打开通知操作失败
                                                            Log.i(TAG, "打开通知操作失败 " + exception.toString());
                                                        }
                                                    });
                                                }

                                                @Override
                                                public void onCharacteristicChanged(final byte[] data) {

                                                    Log.e(TAG, "收到的通知000.。。。 " + HexUtil.formatHexString(data, false));

                                                    runOnUiThread(new Runnable() {
                                                        @Override
                                                        public void run() {
                                                            String testValueData = HexUtil.formatHexString(data, false);

                                                            // 打开通知后，设备发过来的数据将在这里出现
                                                            Log.e(TAG, "收到的通知 " + HexUtil.formatHexString(characteristic.getValue(), false));
                                                            Log.e(TAG, "收到的通知000.。。。2222.。。。 " + HexUtil.formatHexString(data, false));
                                                            //两个厂家返回的数据不一样要做区分  三诺
                                                            if (singleDevice.getWriteType() == BluetoothGattCharacteristic.PROPERTY_WRITE_NO_RESPONSE) {

                                                                if (testValueData.length() == 38 || testValueData.length() == 40) {
                                                                    //包含534e 说明数据正确
                                                                    if (testValueData.contains("534e")) {

                                                                        int location = testValueData.indexOf("53");
                                                                        if (location != 0) {
                                                                            testValueData = testValueData.substring(location, testValueData.length());
                                                                        }
                                                                        if (testValueData.length() == 38 || testValueData.length() == 40) {
                                                                            if (testValueData.contains("04") || testValueData.contains("0e")) {

                                                                                if (testValueData.substring(10, 12).equals("04")) { //04代表当前测试值
                                                                                    testValueData = testValueData.substring(24, 28);//看协议对应位置

                                                                                    //随机数
                                                                                    int numcode = (int) ((Math.random() * 9 + 1) * 1000);
                                                                                    //12FF10FFF3000001007b0100004ce173c4ccec  //测试数值 注意最后六位是Mac
                                                                                    String data = "12FF10FFF300" + numcode + testValueData + "010000" + (bleDevice.getMac()).replace(":", "");
                                                                                    //发数据
                                                                                    serviceBinder.setdata(data);
                                                                                    return;

                                                                                } else if (testValueData.substring(10, 12).equals("0e")) { //0e代表历史记录数据
                                                                                          //历史数据暂时不要 不上传
//                                                                                    testValueData = testValueData.substring(28, 32);//看协议对应位置
//////                                                                                   //随机数
//                                                                                    int numcode = (int) ((Math.random() * 9 + 1) * 1000);
////                                                                                     //12FF10FFF3000001007b0100004ce173c4ccec  //测试数值 注意最后六位是Mac
//                                                                                    String data = "12FF10FFF300" + numcode + testValueData + "010000" + (bleDevice.getMac()).replace(":", "");
////                                                                                      //发数据
//                                                                                    serviceBinder.setdata(data);
//                                                                                    return;
                                                                                }

                                                                            }
                                                                        } else {
                                                                            Log.e(TAG, "数据长度不对，数据解析失败");
                                                                        }

                                                                    }
                                                                }

//                                                                if (datas.length() == 40) {
//                                                                    Log.e(TAG, "datas" + datas);
//                                                                    Log.e(TAG, "收到的通知111.。。。" + HexUtil.formatHexString(characteristic.getValue(), false));
//                                                                    if (!HexUtil.formatHexString(characteristic.getValue(), false).contains("534e")) {
//                                                                        datas = datas + HexUtil.formatHexString(characteristic.getValue(), false);
//
//                                                                        Log.e(TAG, "datas222。。。" + datas);
//                                                                        if (datas.length() == 42) {
//                                                                            String bloodGlucoseLevel = datas.substring(24, 28);
//                                                                            //随机数
//                                                                            int numcode = (int) ((Math.random() * 9 + 1) * 1000);
//                                                                            //12FF10FFF3000001007b0100004ce173c4ccec  //测试数值 注意最后六位是Mac
//                                                                            String data = "12FF10FFF300" + numcode + bloodGlucoseLevel + "010000" + (bleDevice.getMac()).replace(":", "");
//                                                                            //发数据
//                                                                            serviceBinder.setdata(data);
//                                                                            return;
//                                                                        } else {
//                                                                            Log.e(TAG, "组装后的数据长度不对，数据解析失败");
//                                                                        }
//                                                                    } else {
//                                                                        Log.e(TAG, "数据解析失败");
//                                                                    }
//                                                                }

                                                            } else if (singleDevice.getWriteType() == BluetoothGattCharacteristic.PROPERTY_WRITE) { //艾科
                                                                if (datas.length() == 40) {
                                                                    if (HexUtil.formatHexString(characteristic.getValue(), false).contains("0d")) {
                                                                        datas = datas + HexUtil.formatHexString(characteristic.getValue(), false);
                                                                        if (datas.length() == 58) {
                                                                            String bloodGlucoseLevel = datas.substring(36, 42);
                                                                            //十六进制转字符串 在换算出血糖值
                                                                            bloodGlucoseLevel = BluetoothUtil.hexStringToString(bloodGlucoseLevel);
                                                                            bloodGlucoseLevel = String.format("%.1f", ((Integer.getInteger(bloodGlucoseLevel) / 18) + 0.05 - 0.05) * 10);
                                                                            bloodGlucoseLevel = BluetoothUtil.hex10To16(bloodGlucoseLevel);
                                                                            //随机数
                                                                            int numcode = (int) ((Math.random() * 9 + 1) * 1000);
                                                                            //12FF10FFF3000001007b0100004ce173c4ccec //测试数值 注意最后六位是Mac
                                                                            String data = "12FF10FFF300" + numcode + bloodGlucoseLevel + "010000" + (bleDevice.getMac()).replace(":", "");
                                                                            serviceBinder.setdata(data);
                                                                            return;
                                                                        } else {
                                                                            Log.e(TAG, "组装后的数据长度不对，数据解析失败");
                                                                        }


                                                                    }

                                                                }
                                                            }
                                                            datas = HexUtil.formatHexString(characteristic.getValue(), false);
                                                            Log.e(TAG, "收到的通知333.。。。 " + datas);
                                                            Log.e(TAG, "收到的通知000.。。。333333.。。。 " + HexUtil.formatHexString(data, false));
                                                        }
                                                    });
                                                }
                                            });

                                }
                            }
                        }


                    }
                }

            }

            @Override
            public void onDisConnected(boolean isActiveDisConnected, BleDevice bleDevice, BluetoothGatt gatt, int status) {
                progressDialog.dismiss();

                mDeviceAdapter.removeDevice(bleDevice);
                mDeviceAdapter.notifyDataSetChanged();

                if (isActiveDisConnected) {
                    Toast.makeText(MainActivity.this, getString(R.string.active_disconnected), Toast.LENGTH_LONG).show();
                } else {
                    Toast.makeText(MainActivity.this, getString(R.string.disconnected), Toast.LENGTH_LONG).show();
                    ObserverManager.getInstance().notifyObserver(bleDevice);
                }

            }
        });
    }

    @Override
    public void onBackPressed() {
        if (System.currentTimeMillis() - backClick > 2000) {
            Toast.makeText(this, "再按一次退出", Toast.LENGTH_SHORT).show();
        } else {
            super.onBackPressed();
        }
        backClick = System.currentTimeMillis();
    }

    @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.e(TAG, "onDestroy() 守护之影关闭");
        //关闭通知
        if (singleDevice.getCharacteristic() != null) {
            BleManager.getInstance().stopNotify(
                    singleDevice.getDevice(),
                    singleDevice.getCharacteristic().getService().getUuid().toString(),
                    singleDevice.getCharacteristic().getUuid().toString());
        }
        if (BluetoothServiceIntent != null) {
//            stopService(BluetoothServiceIntent);
            unbindService(service);//将service与activity解绑
        }
        if(timer != null){
            timer.cancel();
        }
        if(timerTask != null){
            timerTask.cancel();
        }


    }

    /**
     * 重启蓝牙服务
     */
    public void restartService() {
        unbindService(service);//将service与activity解绑
        BluetoothServiceIntent = new Intent(this, BluetoothService.class);
        //连接服务
        service = new Service();
        bindService(BluetoothServiceIntent, service, BIND_AUTO_CREATE);
    }


    @Override
    protected void Permission() {
        Log.e(TAG, "Permission() 获取到全部权限");
//        BluetoothServiceIntent = new Intent(this, BluetoothService.class);
//        startService(BluetoothServiceIntent);

        BluetoothServiceIntent = new Intent(this, BluetoothService.class);
        //连接服务
        service = new Service();
        bindService(BluetoothServiceIntent, service, BIND_AUTO_CREATE);

    }


    private class Service implements ServiceConnection {
        @Override
        public void onServiceConnected(ComponentName componentName, IBinder iBinder) {
            serviceBinder = (BluetoothService.ServiceBinder) iBinder;
        }

        @Override
        public void onServiceDisconnected(ComponentName componentName) {

        }
    }


    /**
     * 设置域名和ip
     *
     * @param data
     */
    @Override
    public void OnDomainNameTransferIP(final String[] data) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (data == null || data.length == 0) {
                    Toast.makeText(MainActivity.this, "设置失败，域名解析失败", Toast.LENGTH_LONG).show();
                    return;
                }
                int port;
                try {
                    port = Integer.parseInt(portStr);
                } catch (Exception e) {
                    e.printStackTrace();
                    Toast.makeText(MainActivity.this, "设置失败，端口号输入有误", Toast.LENGTH_LONG).show();
                    return;
                }

                NetFunctionConfig.setIP(data[0] + "");
                NetFunctionConfig.setPort(port);
                configuration_layout.setVisibility(View.GONE);
            }
        });


    }
}
