package com.example.bluetoothgateway.base;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.Settings;
import android.util.Log;
import android.view.KeyEvent;
import android.widget.PopupMenu;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.app.NotificationManagerCompat;
import androidx.core.content.ContextCompat;

import com.example.bluetoothgateway.broadcastreceiver.NetBroadcastReceiver;
import com.example.bluetoothgateway.utils.NetFunctionConfig;
import com.example.bluetoothgateway.utils.NetUtil;
import com.permissionx.guolindev.PermissionX;
import com.permissionx.guolindev.callback.ExplainReasonCallback;
import com.permissionx.guolindev.callback.ForwardToSettingsCallback;
import com.permissionx.guolindev.callback.RequestCallback;
import com.permissionx.guolindev.request.ExplainScope;
import com.permissionx.guolindev.request.ForwardScope;

import java.util.List;

import static android.app.Notification.EXTRA_CHANNEL_ID;
import static android.provider.Settings.EXTRA_APP_PACKAGE;
import static com.example.bluetoothgateway.MyApplication.getInstance;

public abstract class BaseActivity extends AppCompatActivity implements NetBroadcastReceiver.NetChangeListener {

    private  String TAG = BaseActivity.class.getSimpleName();
    /**
     * 网络类型
     */
    private int netType;
    public static NetBroadcastReceiver.NetChangeListener listener;
    private NetBroadcastReceiver netBroadcastReceiver;
    private AlertDialog dialogs;
    private BluetoothAdapter bluetoothAdapter;
    private static final int REQUEST_ENABLE_BT = 1;
    AlertDialog alertDialog;
    private boolean permission = false;//为true的时候 说明所有权限已经获取到了
    private long time = 0;
    private OnCloseDialogListener onCloseDialogListener;
    private String [] allPermissions;
    private boolean requesePer = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //动态注册网络监听广播
        IntentFilter filter = new IntentFilter();
        filter.addAction("android.net.conn.CONNECTIVITY_CHANGE");
        listener = this;
        netBroadcastReceiver = new NetBroadcastReceiver(listener);
        registerReceiver(netBroadcastReceiver, filter);

        //bluetoothPermissions();

        bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        requestPermissions();


        checkNet();

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        // User chose not to enable Bluetooth.
        if (requestCode == REQUEST_ENABLE_BT && resultCode == Activity.RESULT_CANCELED) {
            finish();
            return;
        }else {
            //applyPermission();
            requestPermissions();

        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    private void requestPermissions() {
        if (requesePer) {
            return;
        }

        if(Build.VERSION.SDK_INT >=Build.VERSION_CODES.Q){
            allPermissions = allPermissions2;
        }else {
            allPermissions = allPermissions1;
        }

        requesePer = true;
        PermissionX.init(this)
                .permissions(allPermissions1)
                .onExplainRequestReason(new ExplainReasonCallback() {
                    @Override
                    public void onExplainReason(@NonNull ExplainScope scope, @NonNull List<String> deniedList) {
                        scope.showRequestReasonDialog(deniedList, "需要允许权限才能正常使用", "确定");
                    }
                })
                .onForwardToSettings(new ForwardToSettingsCallback() {
                    @Override
                    public void onForwardToSettings(@NonNull ForwardScope scope, @NonNull List<String> deniedList) {
                        scope.showForwardToSettingsDialog(deniedList, "您需要去设置中手动开启相关权限", "确定");
                    }
                })
                .request(new RequestCallback() {
                    @Override
                    public void onResult(boolean allGranted, @NonNull List<String> grantedList, @NonNull List<String> deniedList) {
                        if (allGranted) {
                            Toast.makeText(BaseActivity.this, "所有权限已开启", Toast.LENGTH_LONG).show();
                            /*if (bluetoothAdapter != null && !bluetoothAdapter.isEnabled()) {
                                Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                                startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
                            }*/
                            bluetoothPermissions();
                        } else {
                            Toast.makeText(BaseActivity.this, "部分权限未开启", Toast.LENGTH_LONG).show();
                        }
                    }
                });
    }


    /**
     * 蓝牙权限
     */
    private void bluetoothPermissions() {
        this.bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if (bluetoothAdapter != null) {
            if (!bluetoothAdapter.isEnabled()) {
                Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
            }else {
                //蓝牙打开直接走这里
                //applyPermission();
                permission = true;
                GPSPermission();
            }
        }
    }

    @Override
    protected void onResume() {
        super.onResume();

        /**
         * 跳转到通知界面
         */
        NotificationManagerCompat manager = NotificationManagerCompat.from(this);
        // areNotificationsEnabled方法的有效性官方只最低支持到API 19，低于19的仍可调用此方法不过只会返回true，即默认为用户已经开启了通知。
        boolean isOpened = manager.areNotificationsEnabled();
        if (!isOpened) {
            try {
                // 根据isOpened结果，判断是否需要提醒用户跳转AppInfo页面，去打开App通知权限
                Intent intent = new Intent();
                intent.setAction(Settings.ACTION_APP_NOTIFICATION_SETTINGS);
                //这种方案适用于 API 26, 即8.0（含8.0）以上可以用
                intent.putExtra(EXTRA_APP_PACKAGE, getPackageName());
                intent.putExtra(EXTRA_CHANNEL_ID, getApplicationInfo().uid);

                //这种方案适用于 API21——25，即 5.0——7.1 之间的版本可以使用
                intent.putExtra("app_package", getPackageName());
                intent.putExtra("app_uid", getApplicationInfo().uid);

                // 小米6 -MIUI9.6-8.0.0系统，是个特例，通知设置界面只能控制"允许使用通知圆点"——然而这个玩意并没有卵用，我想对雷布斯说：I'm not ok!!!
                //  if ("MI 6".equals(Build.MODEL)) {
                //      intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                //      Uri uri = Uri.fromParts("package", getPackageName(), null);
                //      intent.setData(uri);
                //      // intent.setAction("com.android.settings/.SubSettings");
                //  }
                startActivity(intent);
            } catch (Exception e) {
                e.printStackTrace();
                // 出现异常则跳转到应用设置界面：锤子坚果3——OC105 API25
                Intent intent = new Intent();

                //下面这种方案是直接跳转到当前应用的设置界面。
                //https://blog.csdn.net/ysy950803/article/details/71910806
                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                Uri uri = Uri.fromParts("package", getPackageName(), null);
                intent.setData(uri);
                startActivity(intent);
            }

        }else {
            Log.e(TAG,"onResume().....");
                GPSPermission();
        }

    }

    /**
     * 初始化时判断有没有网络
     */
    public boolean checkNet() {
        this.netType = NetUtil.getNetWorkState(this);
        if (!isNetConnect()) {
            //网络异常，请检查网络
            showNetDialog();
            Toast.makeText(this, "网络异常，请检查网络", Toast.LENGTH_LONG).show();
        }
        return isNetConnect();
    }

    String[] allPermissions1 = new String[]{
            Manifest.permission.CAMERA,
            Manifest.permission.READ_EXTERNAL_STORAGE,
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.ACCESS_COARSE_LOCATION,
//            Manifest.permission.ACCESS_BACKGROUND_LOCATION //安卓10才支持 低于10的版本会报错
    };

    String[] allPermissions2 = new String[]{
            Manifest.permission.CAMERA,
            Manifest.permission.READ_EXTERNAL_STORAGE,
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.ACCESS_COARSE_LOCATION,
            Manifest.permission.ACCESS_BACKGROUND_LOCATION //安卓10才支持 低于10的版本会报错
    };


    /**
     * 动态获取权限
     */
    public void applyPermission() {
        if (Build.VERSION.SDK_INT >= 23) {
            if(Build.VERSION.SDK_INT >=Build.VERSION_CODES.Q){
              allPermissions = allPermissions2;
            }else {
                allPermissions = allPermissions1;
            }

            boolean needapply = false;
            for (int i = 0; i < allPermissions.length; i++) {
                int chechpermission = ContextCompat.checkSelfPermission(getApplicationContext(), allPermissions[i]);
                if (chechpermission != PackageManager.PERMISSION_GRANTED) {
                    needapply = true;
                }
            }
            if (needapply) {
                ActivityCompat.requestPermissions(this, allPermissions, 1);
            } else {
                permission = true;
                GPSPermission();
            }
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR1)
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        for (int i = 0; i < grantResults.length; i++) {
            if (grantResults[i] == PackageManager.PERMISSION_GRANTED) {
                if (i == (grantResults.length) - 1) {
                    permission = true;
                    GPSPermission();
                }

            } else {
                /*if(i == (grantResults.length) - 1){
                    //Toast.makeText(this, "权限未获取", Toast.LENGTH_LONG).show();
                    alertDialog = new AlertDialog.Builder(this)
                            .setTitle("功能受限")
                            .setMessage("由于尚未授予完全的位置访问权限，可能无法发现设备，如需要重新授权请点击确定进入应用管理界面授权！")
                            .setOnKeyListener(keylistener)
                            .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    dialog.dismiss();

                                    Intent intent = new Intent();
                                    intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                    Uri uri = Uri.fromParts("package", getPackageName(), null);
                                    intent.setData(uri);
                                    startActivity(intent);

                                    //finish();
                                    //onCloseDialogListener.OnCloseDialog(alertDialog);
                                }
                            })
                            .setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    dialog.dismiss();
                                }
                            })
                            .setOnDismissListener(new DialogInterface.OnDismissListener() {
                                @Override
                                public void onDismiss(DialogInterface dialog) {

                                }
                            })
                            .show();
                    alertDialog.setCanceledOnTouchOutside(false);
                }*/
            }
        }
    }

    private void GPSPermission(){
//        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q){
            LocationManager locationManager = (LocationManager)getInstance().getSystemService(LOCATION_SERVICE);
            boolean isGPS = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
            if (!isGPS) {
                Intent intent = new Intent();
                intent.setAction(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                getInstance().startActivity(intent);
            }else {
                if(permission&&(System.currentTimeMillis()-time)/1000>1){
                    Log.e(TAG,"Permission().....");
                    time = System.currentTimeMillis();
                    Permission();
                }

            }
//        }
    }

    protected void requestRunTimePermission(OnCloseDialogListener onCloseDialogListener) {
        this.onCloseDialogListener = onCloseDialogListener;

    }


    protected abstract void Permission();//有权限才回调

    private void showNetDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("网络异常");
        builder.setNegativeButton("取消", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {

            }
        });
        builder.setPositiveButton("设置", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                Intent intent = new Intent(Settings.ACTION_WIFI_SETTINGS);
                startActivity(intent);
            }
        });
        dialogs = builder.show();
    }

    /**
     * 网络变化之后的类型
     */
    @Override
    public void onChangeListener(int netMobile) {
        // TODO Auto-generated method stub
        this.netType = netMobile;
        Log.i("netType", "netType:" + netMobile);
        if (!isNetConnect()) {
            showNetDialog();
            Toast.makeText(this, "网络异常，请检查网络", Toast.LENGTH_LONG).show();
        } else {
            hideNetDialog();
            int port = NetFunctionConfig.getPort();
            Toast.makeText(this, "网络正常: 端口" + port, Toast.LENGTH_LONG).show();
        }
    }

    /**
     *
     */
    private void hideNetDialog() {
        if (dialogs != null) {
            dialogs.dismiss();
            dialogs.cancel();
        }
    }

    /**
     * 判断有无网络 。
     *
     * @return true 有网, false 没有网络.
     */
    public boolean isNetConnect() {
        if (netType == 2) {
            return true;
        } else if (netType == 1) {
            return true;
        } else if (netType == 0) {
            return true;
        } else if (netType == -1) {
            return false;
        }
        return false;
    }

    DialogInterface.OnKeyListener keylistener = new DialogInterface.OnKeyListener() {
        public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event) {
            if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0) {
                return true;
            } else {
                return false;
            }
        }
    };


    public interface OnCloseDialogListener{
       void OnCloseDialog(AlertDialog alertDialog);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (netBroadcastReceiver != null) {
            unregisterReceiver(netBroadcastReceiver);
        }
    }


}
