package com.example.bluetoothgateway.net;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.app.TaskStackBuilder;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.le.ScanFilter;
import android.bluetooth.le.ScanSettings;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.os.Binder;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;
import android.util.SparseArray;

import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.core.app.NotificationCompat;

import com.example.bluetoothgateway.MainActivity;
import com.example.bluetoothgateway.R;
import com.example.bluetoothgateway.net.impl.BaxcSocketConnectLine;
import com.example.bluetoothgateway.utils.BluetoothUtil;
import com.example.bluetoothgateway.utils.NetFunctionConfig;
import com.example.bluetoothgateway.utils.NotificationUtil;

import org.altbeacon.beacon.BeaconConsumer;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import static com.example.bluetoothgateway.utils.BluetoothUtil.bytesToHex;

public class BluetoothService extends Service implements BeaconConsumer, SocketConnect.OnConnectStateListener, SocketConnect.OnSocketReadListener {

    private String TAG = BluetoothService.class.getSimpleName();
    private BluetoothAdapter bluetoothAdapter;

    private static final int DATA_TYPE_FLAGS = 0x01;
    private static final int DATA_TYPE_SERVICE_UUIDS_16_BIT_PARTIAL = 0x02;
    private static final int DATA_TYPE_SERVICE_UUIDS_16_BIT_COMPLETE = 0x03;
    private static final int DATA_TYPE_SERVICE_UUIDS_32_BIT_PARTIAL = 0x04;
    private static final int DATA_TYPE_SERVICE_UUIDS_32_BIT_COMPLETE = 0x05;
    private static final int DATA_TYPE_SERVICE_UUIDS_128_BIT_PARTIAL = 0x06;
    private static final int DATA_TYPE_SERVICE_UUIDS_128_BIT_COMPLETE = 0x07;
    private static final int DATA_TYPE_LOCAL_NAME_SHORT = 0x08;
    private static final int DATA_TYPE_LOCAL_NAME_COMPLETE = 0x09;
    private static final int DATA_TYPE_TX_POWER_LEVEL = 0x0A;
    private static final int DATA_TYPE_SERVICE_DATA = 0x16;
    private static final int DATA_TYPE_MANUFACTURER_SPECIFIC_DATA = 0xFF;

    SparseArray<byte[]> manufacturerData = new SparseArray<byte[]>();
    private String localName = "";
    private String ibeaconName = "";
    private String address = "";
    private String uuid = "";
    private String majors = "";
    private String minors = "";
    private String rssis = "";
    private String distances = "";
    private int txPowerLevel = Integer.MIN_VALUE;
    private String referenceData = "12FF10FF";//过滤我们的协议头
    private String bloodPressureMeterDtat = "0000000000";//过滤血压仪数据
    private String bloodGlucoseData = "F2";  //过滤血氧仪数据
    private BaxcSocketConnectLine baxcSocketConnectLine;
    private String ip = "47.115.176.250";
    private int port = 3006;
    private static BluetoothService bluetoothService;
    public static boolean connectState = false;
    private List<String> dataStringList = new ArrayList<>();
    private long recordTime = 0;
    private int SCREEN_TIME = 30;


    public static final int NOTICE_ID = 100;
    public static final String CHANNEL_ID_STRING = "nyd001";
    public static String app_name = "APP";

    private long time = 0;
    NotificationUtil notificationUtil;


    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return new ServiceBinder();
    }

    public class ServiceBinder extends Binder {
        public BluetoothService getService() {
            return BluetoothService.this;
        }

        public void setdata(String data) {
            sendData(data);
        }

    }


    @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
    @Override
    public void onCreate() {
        super.onCreate();

//        Context mContext;
//        NotificationManager notificationManager;
//        int progress = 0;
//        int notificationId = 0x2154;
//        Notification.Builder builder;
//        String ContentTitle = "守护之影正在运行";
//        String ContentText = "哈哈哈";
//
//        notificationManager = (NotificationManager) this.getSystemService(NOTIFICATION_SERVICE);
//        String channelId = "my_channel_01";
//        String name = "我是渠道名字";
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
//            NotificationChannel mChannel = new NotificationChannel(channelId, name, NotificationManager.IMPORTANCE_LOW);
//            notificationManager.createNotificationChannel(mChannel);
//            builder = new Notification.Builder(getApplication())
//                    .setChannelId(channelId)  //设置渠道id
//                    .setContentTitle(ContentTitle)
//                    .setContentText(ContentText)
//                    .setSmallIcon(R.mipmap.app_logo);
//        } else {
//            builder = new Notification.Builder(getApplication())
//                    .setContentTitle(ContentTitle)
//                    .setContentText(ContentText)
//                    .setSmallIcon(R.mipmap.app_logo)
//                    .setOngoing(true);
////                    .setChannelId(channelId);//无效
//        }
//        //创建点击跳转Intent
//        Intent intent = new Intent(this, MainActivity.class);
//        //创建任务栈Builder
//        TaskStackBuilder stackBuilder = TaskStackBuilder.create(this);
//        stackBuilder.addParentStack(MainActivity.class);
//        stackBuilder.addNextIntent(intent);
//        PendingIntent pendingIntent = stackBuilder.getPendingIntent(0,PendingIntent.FLAG_UPDATE_CURRENT);
//        //设置跳转Intent到通知中
//        builder.setContentIntent(pendingIntent);
//
////        notificationManager.notify(notificationId, builder.build());
//        startForeground(notificationId, builder.build());//8.0之后Service里面启动Service一定要加这个


        //启动通知栏
        notificationUtil = new NotificationUtil(this);
        notificationUtil.ceShi();
        startForeground(notificationUtil.notificationId, notificationUtil.getBuilder().build());
        //启动蓝牙扫描
        this.bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if (!this.bluetoothAdapter.isEnabled()) {
            this.bluetoothAdapter.enable();
        }
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
//            bluetoothAdapter.getBluetoothLeScanner()
//                    .startScan(Collections.singletonList(new ScanFilter.Builder().build()), new ScanSettings.Builder().build(), new BluetoothAdapter().getBluetoothLeScanner());
//        }
        this.bluetoothAdapter.startLeScan(new BluetoothAdapter.LeScanCallback() {
            @Override
            public void onLeScan(BluetoothDevice device, int rssi, byte[] scanRecord) {
                time = System.currentTimeMillis();

                localName = "";
                ibeaconName = "";
                address = "";
                uuid = "";
                majors = "";
                minors = "";
                rssis = "";
                distances = "";
                txPowerLevel = Integer.MIN_VALUE;

//            customParseData(device, rssi, scanRecord);//自定义解析数据 给蓝牙信标使用
                parseData(device, rssi, scanRecord);//标准蓝牙协议解析数据

            }
        });

        ip = NetFunctionConfig.getIp();
        port = NetFunctionConfig.getPort();
        Log.e(TAG, "ip " + ip + " port " + port);
        if (ip == null || port == 0) {
            NetFunctionConfig.setIP("47.115.176.250");
            NetFunctionConfig.setPort(3006);
            ip = "47.115.176.250";
            port = 3006;
        }
        bluetoothService = this;
        baxcSocketConnectLine = new BaxcSocketConnectLine();
        baxcSocketConnectLine.setIp(ip);
        baxcSocketConnectLine.setPort(port);

        baxcSocketConnectLine.setHeartbeatTime(1000 * 10);
//        String data = String.format("0-%s:%s:%s",  NetFunctionConfig.getUser() + "", BaxcWebConfig.MACHINE_TYPE, Util.getDeviceId(WebSocketClinetService2.this)); //机构ID:机器类型:机器码
        final String data = "12345678";
        baxcSocketConnectLine.setHeartbeat(BluetoothUtil.hexStringToByteArray(data));//发送心跳数据
        baxcSocketConnectLine.start();
        baxcSocketConnectLine.setOnConnectStateListener(this);//监听是否在线
        baxcSocketConnectLine.setOnSocketReadListener(this);  //监听返回数据

    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Log.e(TAG, "onStartCommand....");

        return super.onStartCommand(intent, flags, startId);
    }

    /**
     * 自定义解析数据 只给信标使用
     *
     * @param device
     * @param rssi
     * @param scanRecord
     */
    private void customParseData(BluetoothDevice device, int rssi, byte[] scanRecord) {
        int startIndex = 2;
        boolean patternFound = false;
        Log.d(TAG, "handleScanResult: " + bytesToHex(scanRecord));
        // 寻找是否存在beacon以及有效数据的起始索引
        while (startIndex <= 5) {
//            Log.e(TAG,"onLeScan 走了。。。"+startIndex);
            if (((int) scanRecord[startIndex + 2] & 0xff) == 0x02
                    && ((int) scanRecord[startIndex + 3] & 0xff) == 0x15) {
                patternFound = true;
//                Log.e(TAG,"onLeScan 走了222。。。");
                break;
            }
            startIndex++;
        }
//        Log.e(TAG,"onLeScan 走了333。。。"+startIndex);

        // 如果找到
        if (patternFound) {
            ibeaconName = device.getName();
            address = device.getAddress();
            Log.d(TAG, "customParseData(): 它的名字是" + ibeaconName);
            Log.d(TAG, "customParseData(): 搜索到一个Beacon设备" + address);
            parseBLEData(scanRecord, startIndex);
            double distance = BluetoothUtil.rssi2distance(rssi);
            distances = String.format("%.4f", distance);
            Log.d(TAG, "customParseData(): RSSI=" + rssi + " distances " + distances);

        } else {
            String name = device.getName();
            if (name == null) {
                //Log.d(TAG, "onLeScan: 搜索到一个普通蓝牙设备，它的MAC是"+device.getAddress());
            } else {
                //Log.d(TAG, "onLeScan: 搜索到一个普通蓝牙设备，它的名字是"+name);
            }
        }

    }

    /**
     * 标准蓝牙协议解析数据
     *
     * @param device
     * @param rssi
     * @param scanRecord
     */
    private void parseData(BluetoothDevice device, int rssi, byte[] scanRecord) {


        if (recordTime == 0) {
            recordTime = System.currentTimeMillis();
        }
        int currentPos = 0;
        int advertiseFlag = -1;
        if (scanRecord == null) {
            return;
        }
        try {
            while (currentPos < scanRecord.length) {
                int length = scanRecord[currentPos++] & 0xFF;
                if (length == 0) {
                    break;
                }
                Log.e(TAG, "蓝牙原始数据 " + bytesToHex(scanRecord));
                int dataLength = length - 1;
//                Log.e(TAG, "currentPos " + currentPos);
//                Log.e(TAG, "length " + length);
//                Log.e(TAG, "dataLength " + dataLength);
                int fieldType = scanRecord[currentPos++] & 0xFF;
//                Log.e(TAG, "fieldType " + fieldType + " DATA_TYPE_FLAGS " + DATA_TYPE_FLAGS);
                switch (fieldType) {
                    case DATA_TYPE_FLAGS:
                        advertiseFlag = scanRecord[currentPos] & 0xFF;
                        break;
                    case DATA_TYPE_SERVICE_UUIDS_16_BIT_PARTIAL:
                    case DATA_TYPE_SERVICE_UUIDS_16_BIT_COMPLETE:

                        break;
                    case DATA_TYPE_SERVICE_UUIDS_32_BIT_PARTIAL:
                    case DATA_TYPE_SERVICE_UUIDS_32_BIT_COMPLETE:
                        break;
                    case DATA_TYPE_SERVICE_UUIDS_128_BIT_PARTIAL:
                    case DATA_TYPE_SERVICE_UUIDS_128_BIT_COMPLETE:
                        break;
                    case DATA_TYPE_LOCAL_NAME_SHORT:
                    case DATA_TYPE_LOCAL_NAME_COMPLETE:
                        localName = new String(
                                BluetoothUtil.extractBytes(scanRecord, currentPos, dataLength));
                        break;
                    case DATA_TYPE_TX_POWER_LEVEL:
                        txPowerLevel = scanRecord[currentPos];
                        break;
                    case DATA_TYPE_SERVICE_DATA:
                        break;
                    case DATA_TYPE_MANUFACTURER_SPECIFIC_DATA:
                        // The first two bytes of the manufacturer specific data are
                        // manufacturer ids in little endian.
                        int manufacturerId = ((scanRecord[currentPos + 1] & 0xFF) << 8) +
                                (scanRecord[currentPos] & 0xFF);
                        final byte[] manufacturerDataBytes = BluetoothUtil.extractBytes(scanRecord, currentPos - 2,
                                dataLength + 2);

                        String data = bytesToHex(manufacturerDataBytes);
                        String headData = data.substring(0, 8);
                        if (referenceData.equals(headData)) {
                            String serialNumber = data.substring(10, 20);//血压仪过滤
                            String bloodGlucose = data.substring(8, 10);//血糖仪过滤
                            Log.e(TAG, "发送的数据头部 " + headData);
                            Log.e(TAG, "血压仪数据 " + serialNumber);
                            Log.e(TAG, "血糖仪数据 " + bloodGlucose);
                            if (!bloodPressureMeterDtat.equals(serialNumber)) {
                                if (bloodGlucoseData.equals(bloodGlucose)) {
                                    data = data.substring(0, data.length() - 2);
                                }
//                                Log.e(TAG,"时间 "+ (System.currentTimeMillis()-recordTime)/1000);
                                if ((System.currentTimeMillis() - recordTime) / 1000 <= SCREEN_TIME) {

                                    if (!dataStringList.contains(data)) {
                                        dataStringList.add(data);
                                        Log.e(TAG,"发送的数据 mac:"+device.getAddress());
                                        Log.e(TAG, "发送的数据 " + data);
//                                    sendData(data);
//                                    String SS ="3418AA550000002B0000010000DB5755";
                                        new Thread(new Runnable() {
                                            @Override
                                            public void run() {
                                                baxcSocketConnectLine.send(manufacturerDataBytes);
                                            }
                                        }).start();

                                    }
                                } else {
                                    Log.e(TAG, "时间到了。。。 ");
                                    dataStringList.clear();
                                    recordTime = 0;
                                }
                            }
                        }

                        break;
                    default:
                        // Just ignore, we don't handle such data type.
                        break;
                }
                currentPos += dataLength;
//                Log.e(TAG, "------ " + currentPos);
            }

        } catch (Exception e) {
            Log.e(TAG, "解析蓝牙数据异常 " + e.getMessage());
        }

    }

    // 解析BLE数据
    public void parseBLEData(byte[] scanRecord, int startIndex) {
        // uuid的长度是16bytes
        byte[] uuidBytes = new byte[16];
        System.arraycopy(scanRecord, startIndex + 4, uuidBytes, 0, 16);
        String hexString = bytesToHex(uuidBytes);
        // beacon的UUID值
        uuid = BluetoothUtil.parseUUID(hexString);
        // beacon的Major值
        int major = (scanRecord[startIndex + 20] & 0xff) * 0x100
                + (scanRecord[startIndex + 21] & 0xff);
        majors = major + "";
        // ibeacon的Minor值
        int minor = (scanRecord[startIndex + 22] & 0xff) * 0x100
                + (scanRecord[startIndex + 23] & 0xff);
        minors = minor + "";
        txPowerLevel = (scanRecord[startIndex + 24]);
        Log.d(TAG, "parseBLEData() 它的UUID是" + uuid + "，txPower是" + txPowerLevel);
        Log.d(TAG, "parseBLEData() major=" + major + "，minor=" + minor);
    }

    @Override
    public void onBeaconServiceConnect() {

    }

    /**
     * @param install
     * @param connectState true为连接成功 false为断开连接
     */
    @Override
    public void onConnectState(SocketConnect install, boolean connectState) {
        this.connectState = connectState;
        Log.e(TAG, "连接服务器是否成功 " + connectState);
    }

    /**
     * 服务是否存活
     *
     * @return true: 存活
     */
    public static boolean isServiceRunning() {
        return bluetoothService != null;
    }

    @Override
    public void onSocketRead(SocketConnect install, byte[] bytes) {
//        String data = new String(bytes);

        String data = BluetoothUtil.bytesToHex(bytes);
        Log.e(TAG, "返回的数据data " + data);
        Log.e(TAG, "返回的数据 dd " + (System.currentTimeMillis() - time) / 1000);
        if (time == 0) return;
        if ((System.currentTimeMillis() - time) / 1000 >= 2) {
            time = 0;

//                     直接启动Activity也可以
//                     Intent intent = new Intent(getApplication(),MainActivity.class);
//                     intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
//                     startActivity(intent);

            baxcSocketConnectLine.release();
            notificationUtil.setProgres(notificationUtil.progressMax + 5);

            MainActivity.mainActivity.restartService();

//            Intent iService = new Intent(getApplication(), BluetoothService.class);
//            iService.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
//            getApplication().stopService(iService);
//
//            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
//                getApplication().startForegroundService(new Intent(getApplication(), BluetoothService.class));
//            } else {
//                getApplication().startService(new Intent(getApplication(), BluetoothService.class));
//            }

        }


    }

    /**
     * 发送数据
     *
     * @param data
     */
    public void sendData(final String data) {
        if (baxcSocketConnectLine != null) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    Log.e(TAG, "发送的数据 " + data);
                    baxcSocketConnectLine.send(BluetoothUtil.hexStringToByteArray(data));
                }
            }).start();

        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Log.e(TAG, "蓝牙服务关闭");
//        if(baxcSocketConnectLine != null){
//            baxcSocketConnectLine.release();
//        }
        bluetoothService = null;

    }
}
