package com.example.bluetoothgateway.net;


/**
 * socket连接类 断开连接可自动重连
 * author 许柏胜
 */
public interface SocketConnect {
    int ERROR_START_FAIL = -1;
    int ERROR_RELEASE_FAIL = -1;
    int ERROR_SEND_NOT_CONNECT = -1; //未连接
    int ERROR_SEND_IO_ERROR = -2; //IO异常

    /**
     * 开始连接
     * @return 0为成功 非0为失败
     */
    int start();

    /**
     * 释放资源
     * @return 0为成功 非0为失败
     */
    int release();

    /**
     * 发送数据
     * @param datas
     * @return 小于0为失败
     */
    int send(byte[] datas);

    boolean isConnect();

    void setIp(String ip);
    void setPort(int port);
    void setTag(String tag);

    /**
     * 设置心跳
     * @param heartbeat
     */
    void setHeartbeat(byte[] heartbeat);

    /**
     * 设置心跳间隔时间
     * @param heartbeatTime 默认为5秒
     */
    void setHeartbeatTime(long heartbeatTime);

    long getHeartbeatTime();
    byte[] getHeartbeat();
    String getIp();
    int getPort();
    String getTag();


    void setOnSocketReadListener(OnSocketReadListener onSocketReadListener);

    void setOnConnectStateListener(OnConnectStateListener onConnectStateListener);

    /**
     * 读取回调
     */
    interface OnSocketReadListener{
        void onSocketRead(SocketConnect install, byte[] bytes);
    }

    /**
     * 连接状态监听
     */
    interface OnConnectStateListener{
        /**
         *
         * @param connectState true为连接成功 false为断开连接
         */
        void onConnectState(SocketConnect install, boolean connectState);
    }

}
