package com.example.bluetoothgateway.net.impl;


import android.util.Log;


import com.example.bluetoothgateway.net.SocketConnect;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;

/**
 * socket客户端 读取一行一行数据
 * author 许柏胜
 */
public class BaxcSocketConnectLine implements SocketConnect {

    private String ip;
    private int port;
    private String tag;
    private byte[] heartbeat;
    private long heartbeatTime = 5000;
    private boolean isConnect;
    private OnSocketReadListener onSocketReadListener;
    private OnConnectStateListener onConnectStateListener;

    private ConnectThread connectThread;
    private BufferedReader bufferedReader;
    private OutputStream outputStream;
    private Socket socket;
    private ReadThread readThread;
    private HeartbeatThread heartbeatThread;

    @Override
    public int start() {
        startConnectThread();
        return 0;
    }

    @Override
    public int release() {

        stopConnectThread();
        stopReadThread();
        stopHeartbeatThread();

        if (bufferedReader != null) {
            try {
                bufferedReader.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            bufferedReader = null;
        }

        if (outputStream != null) {
            try {
                outputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            outputStream = null;
        }

        if (socket != null) {
            try {
                socket.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            socket = null;
        }

        isConnect = false;

        onConnectState(false);
        return 0;
    }

    @Override
    public int send(byte[] datas) {
        return sendData(datas);
    }

    /**
     * 连接线程
     */
    class ConnectThread extends Thread {
        private boolean threadBool = false;
        private int timeout = 5000;
        private Socket socketTemp;
        private boolean isConnected = false; //是否正在连接

        @Override
        public void run() {
            super.run();
            threadBool = true;
            isConnected = true;
            while (threadBool) {
                try {
                    socketTemp = new Socket();
                    socketTemp.connect(new InetSocketAddress(ip, port), timeout);
                    isConnected = false;
                    socket = socketTemp;
                    bufferedReader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
                    outputStream = socket.getOutputStream();

                    startReadThread();
                    startHeartbeatThread();

                    log("连接成功");
                    isConnect = true;
                    onConnectState(true);
                    stopThread();
                    break;
                } catch (IOException e) {
                    //e.printStackTrace();
                    log("连接失败重新连接："+ip+":"+port);
                }

                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            log("退出连接线程"+ip+":"+port);
        }

        public void stopThread() {
            threadBool = false;

            Log.e("xxxx","");

            if (isConnected) {
                if (socketTemp != null) {
                    try {
                        socketTemp.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
            interrupt();
        }
    }


    private void log(String str) {
        System.out.println(str);
    }

    private void startConnectThread() {
        stopConnectThread();
        connectThread = new ConnectThread();
        connectThread.start();
    }

    private void stopConnectThread() {
        if (connectThread != null) {
            connectThread.stopThread();
            connectThread = null;
        }
    }



    void onConnectState(boolean connectState) {
        if (onConnectStateListener != null) {
            onConnectStateListener.onConnectState(this, connectState);
        }
    }

    private void startHeartbeatThread() {
        stopHeartbeatThread();
        heartbeatThread = new HeartbeatThread();
        heartbeatThread.start();
    }

    private void stopHeartbeatThread() {
        if (heartbeatThread != null) {
            heartbeatThread.stopThread();
            heartbeatThread = null;
        }
    }

    synchronized public int sendData(byte[] bytes){
        int ret = 0;
        if(bytes == null || bytes.length == 0) return ret;
        try {
            if (outputStream != null) {
                outputStream.write(bytes);
                outputStream.flush();
                ret = 0;
            }else ret = -1;
        } catch (IOException e) {
            e.printStackTrace();
            ret = -2;
        }
        return ret;
    }

    /**
     * 心跳线程
     */
    class HeartbeatThread extends Thread {

        private boolean threadBool = false;

        @Override
        public void run() {
            super.run();
            threadBool = true;
            while (threadBool) {
                Log.e("sss777","心跳线程");
                int ret = sendData(getHeartbeat());
                if(ret != 0){
                    stopThread();
                    //重新连接
                    log("发送失败，重新连接");
                    release();
                    startConnectThread();
                    break;
                }
                try {
                    Thread.sleep(heartbeatTime);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            log("退出心跳线程");
        }

        public void stopThread() {
            threadBool = false;
            interrupt();
        }
    }


    private void startReadThread() {
        stopReadThread();
        readThread = new ReadThread();
        readThread.start();
    }

    private void stopReadThread() {
        if (readThread != null) {
            readThread.stopThread();
            readThread = null;
        }
    }


    /**
     * 读取数据线程
     */
    class ReadThread extends Thread {

        private boolean threadBool = false;

        @Override
        public void run() {
            super.run();
            Log.e("sss","读取数据线程");
            threadBool = true;
            while (threadBool) {
                Log.e("sss111","读取数据线程");
                if (bufferedReader != null) {
                    Log.e("sss222","读取数据线程");
                    try {
                        String str = bufferedReader.readLine();
                        disposeReadData(str.getBytes());
                        Log.e("sss333","读取数据线程");
                    } catch (Exception e) {
                        Log.e("sss44444","读取数据线程");
                        e.printStackTrace();
                        stopThread();
                    }
                }
            }

            log("退出读取数据线程");
        }

        public void stopThread() {
            threadBool = false;
            interrupt();
        }
    }

    private void disposeReadData(byte[] bytes) {
        onSocketRead(bytes);
    }
    private void onSocketRead(byte[] bytes){
        Log.e("sss555","读取数据线程"+onSocketReadListener);
        if(onSocketReadListener != null){
            Log.e("sss666","读取数据线程");
            onSocketReadListener.onSocketRead(this, bytes);
        }
    }

    @Override
    public boolean isConnect() {
        return isConnect;
    }

    @Override
    public void setIp(String ip) {
        this.ip = ip;
    }

    @Override
    public void setPort(int port) {
        this.port = port;
    }

    @Override
    public void setTag(String tag) {
        this.tag = tag;
    }

    @Override
    public void setHeartbeat(byte[] heartbeat) {
        this.heartbeat = heartbeat;
    }

    @Override
    public void setHeartbeatTime(long heartbeatTime) {
        this.heartbeatTime = heartbeatTime;
    }

    @Override
    public long getHeartbeatTime() {
        return heartbeatTime;
    }

    @Override
    public byte[] getHeartbeat() {
        return heartbeat;
    }

    @Override
    public String getIp() {
        return ip;
    }

    @Override
    public int getPort() {
        return port;
    }

    @Override
    public String getTag() {
        return tag;
    }

    @Override
    public void setOnSocketReadListener(OnSocketReadListener onSocketReadListener) {
        this.onSocketReadListener = onSocketReadListener;
    }

    @Override
    public void setOnConnectStateListener(OnConnectStateListener onConnectStateListener) {
        this.onConnectStateListener = onConnectStateListener;
    }
}
