package com.example.bluetoothgateway.utils;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.util.Log;
import android.widget.Toast;

import androidx.core.content.FileProvider;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

public class DownloadAppUtil {

    private String TAG = DownloadAppUtil.class.getSimpleName();
    private static DownloadAppUtil downloadAppUtil;
    private OkHttpClient okHttpClient;
    //下载的APK文件绝对路径
    public static final String FILE_APK_PATH = Environment.getExternalStorageDirectory() + "/WDKLGuardian";
    //url返回的APP版本号
    public static String FILE_APK_VERSIONS = "";
    //当前的版本号
    public static String now = "";
    //保存的版本号
    public static String save = "";
    //下载的apk文件
    public static String FILE_APK_NAME = "守护之影_";
    //当前APP的名称
    public static String NOW_APP_NAME = "守护之影_";

    public static Context mContext;

    public static DownloadAppUtil getInstance(Context context) {
        mContext = context;
        if (downloadAppUtil == null) {
            synchronized (DownloadAppUtil.class) {
                if (downloadAppUtil == null) {
                    downloadAppUtil = new DownloadAppUtil();
                }
            }
        }
        return downloadAppUtil;

    }

    DownloadAppUtil() {
        if (okHttpClient == null) {
            okHttpClient = new OkHttpClient();
        }

    }

    public void downloadApp(final String url, final OnDownloadLitener listener) {
        String data[] = url.split("/");
        String versions = data[data.length - 1];
        FILE_APK_VERSIONS = versions.substring(16, 19);

        Log.e(TAG, " 服务器的版本" + FILE_APK_VERSIONS);

        Request request = new Request.Builder()
                .url(url)
                .build();
        okHttpClient.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                listener.onDownloadFail(e.getMessage());
            }

            @Override
            public void onResponse(Call call, Response response) {
                final String fileName = ExistingOrNot(new File(FILE_APK_PATH),
                        new File(FILE_APK_PATH + "/" + FILE_APK_NAME + FILE_APK_VERSIONS + ".apk"), listener);
                if (fileName == null) {
                    listener.onDownloadFail("app下载异常，文件夹创建失败。");
                    return;
                } else if (fileName.equals("已存在")) {
                    return;
                }
                InputStream is = null;
                byte[] buf = new byte[2048];
                int len;
                FileOutputStream fos = null;
                try {
                    is = response.body().byteStream();
                    long total = response.body().contentLength();
                    File file = new File(fileName, FILE_APK_NAME + FILE_APK_VERSIONS + ".apk");
                    fos = new FileOutputStream(file);
                    long sum = 0;
                    while ((len = is.read(buf)) != -1) {
                        fos.write(buf, 0, len);
                        sum = sum + (long) len;
                        //int progress = (int) (sum * 1.0f / total * 100);
                        float sp = (float) sum / (float) total;
                        int progress = (int) (sp * 100);
                        Log.d(TAG, "app下载中progress==" + progress);
                        listener.onBeDownloading(progress);// 下载中
                    }
                    fos.flush();
                    listener.onDownloadSucceed(""); // 下载完成
                } catch (Exception e) {
                    Log.d(TAG, "APP下载异常Exception==");
                    listener.onDownloadFail("app下载异常 " + e.getMessage());
                } finally {
                    try {
                        if (is != null)
                            is.close();
                        if (fos != null)
                            fos.close();
                    } catch (IOException e) {
                        Log.d(TAG, "IOException==");
                    }
                }
            }
        });

    }

    public String ExistingOrNot(File downloadFile, File sonFile, OnDownloadLitener listener) {
        try {
            if (!downloadFile.mkdirs()) {
                downloadFile.createNewFile();
            }

            //判断APP文件是否存在 在做版本对比
            if (isAPKFile() != 0) {
                Log.e(TAG, "最新APP安装包已存在");
                listener.onDownloadSucceed("");
                return "已存在";
            }
            return downloadFile.getAbsolutePath();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;

    }

    public int isAPKFile() {
        try {
            if(!new File(FILE_APK_PATH).mkdirs())return 0;

            File file = new File(FILE_APK_PATH);
            Log.e(TAG, "文件下的APP数量" + file.listFiles().length);
            if (file.listFiles().length == 1) {
                File files[] = file.listFiles();
                now = Utils.getAppVersionName(mContext);
                save = files[0].getName().substring(5, 8);

                Log.e(TAG, "当前的版本号 " + now + " 下载APP的版本号 " + save);

                if (files[0].getName().equals(NOW_APP_NAME + now + ".apk")) {
                    files[0].delete();
                    return 0;
                } else if (Float.parseFloat(now) < Float.parseFloat(save)) {
                    return 1;
                } else {
                    files[0].delete();
                    return 0;
                }
            } else if (file.listFiles().length > 1) {
                //基本不会有这种情况不处理
            }

        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
        return 0;
    }


    public interface OnDownloadLitener {

        void onDownloadSucceed(String data);

        void onBeDownloading(int progress);

        void onDownloadFail(String data);

    }

}
