package com.example.bluetoothgateway.utils;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.TaskStackBuilder;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;

import com.example.bluetoothgateway.MainActivity;
import com.example.bluetoothgateway.R;
import com.example.bluetoothgateway.net.BluetoothService;

import java.text.SimpleDateFormat;
import java.util.Date;

import static android.content.Context.NOTIFICATION_SERVICE;

/**
 * 通知栏工具
 */
public class NotificationUtil {

    private String TAG = NotificationUtil.class.getSimpleName();
    private Context mContext;
    private NotificationManager notificationManager;
    private int progress = 0;
    public int progressMax = 600;
    public int notificationId = 0x2154;
    private Notification.Builder builder;
    private String ContentTitle = "守护之影正在运行";
    private String ContentText = "运行中";
    String channelId = "my_channel_01";
    String name = "我是渠道名字";

    private SimpleDateFormat sdfInput = new SimpleDateFormat("mm:ss ");

    /**
     * @param context
     */
    public NotificationUtil(Context context) {
        this.mContext = context;
        notificationManager = (NotificationManager) mContext.getSystemService(NOTIFICATION_SERVICE);
        sendNotification();
    }

    public Notification.Builder getBuilder(){
        return builder;
    }

    public void setProgres(int progress){
        this.progress = progress;
    }

    @TargetApi(Build.VERSION_CODES.O)
    private void sendNotification() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            @SuppressLint("WrongConstant")
            NotificationChannel mChannel = new NotificationChannel(channelId, name, NotificationManager.IMPORTANCE_LOW);
            notificationManager.createNotificationChannel(mChannel);
            builder = new Notification.Builder(mContext)
                    .setChannelId(channelId)  //设置渠道id
                    .setContentTitle(ContentTitle)
                    .setContentText(ContentText)
                    .setSmallIcon(R.mipmap.app_logo);
            // 会重新启动一个界面
//            Intent activityIntent = new Intent(mContext, MainActivity.class);
//            PendingIntent pendingIntent = PendingIntent.getActivity(mContext, 1, activityIntent, PendingIntent.FLAG_UPDATE_CURRENT);
//            builder.setContentIntent(pendingIntent);

            //创建点击跳转Intent 不会重新新建界面
            Intent intent = new Intent(mContext, MainActivity.class);
            //创建任务栈Builder
            TaskStackBuilder stackBuilder = TaskStackBuilder.create(mContext);
            stackBuilder.addParentStack(MainActivity.class);
            stackBuilder.addNextIntent(intent);
            PendingIntent pendingIntent = stackBuilder.getPendingIntent(0,PendingIntent.FLAG_UPDATE_CURRENT);
            builder.setContentIntent(pendingIntent);
        } else {
            builder = new Notification.Builder(mContext)
                    .setContentTitle(ContentTitle)
                    .setContentText(ContentText)
                    .setSmallIcon(R.mipmap.app_logo)
                    .setOngoing(true);
//                    .setChannelId(channelId);//无效
        }
        notificationManager.notify(notificationId, builder.build());
    }


    /**
     * @param contentText
     * @return
     */
    public NotificationUtil setContentText(String contentText) {
        ContentText = contentText;
        return NotificationUtil.this;
    }


    /**
     * @param contentTitle
     * @return
     */
    public NotificationUtil setContentTitle(String contentTitle) {
        ContentTitle = contentTitle;
        return NotificationUtil.this;
    }


    /**
     * @param progress
     */
    @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN)
    public void setProgress(int progress) {
        if (builder == null) return;
        builder.setProgress(progressMax, progress, false);
//        builder.setContentTitle("下载中...");
        builder.setContentTitle(sdfInput.format(new Date(progress*1000)));
        if (progress == progressMax) {
            builder.setContentTitle("下载完成");
            //自动移除
            notificationManager.cancel(notificationId);
        } else {
            notificationManager.notify(notificationId, builder.build());//显示通知
        }
    }


    public void ceShi() {
        new Thread(new Runnable() {
            @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN)
            @Override
            public void run() {
                while (progress++ < progressMax) {
                    setProgress(progress);
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }).start();
    }
}