package com.example.bluetoothgateway.utils;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.util.Log;

import com.example.bluetoothgateway.BuildConfig;
import com.example.bluetoothgateway.MainActivity;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Utils {

    private Context context;
    public static OnDomainNameTransferIPListener onDomainNameTransferIPListener;


    public Utils(OnDomainNameTransferIPListener onDomainNameTransferIPListener,Context context){
        this.onDomainNameTransferIPListener = onDomainNameTransferIPListener;
        this.context = context;
    }


    /**
     * 返回当前程序版本号
     */
    public static String getAppVersionCode(Context context) {
        int versioncode = 0;
        try {
            PackageManager pm = context.getPackageManager();
            PackageInfo pi = pm.getPackageInfo(context.getPackageName(), 0);
            // versionName = pi.versionName;
            return pi.versionCode + "";
        } catch (Exception e) {
            Log.e("VersionInfo", "Exception", e);
        }
        return versioncode + "版本号获取失败";
    }

    /**
     * 返回当前程序版本名
     */
    public static String getAppVersionName(Context context) {
        String versionName = null;
        try {
            PackageManager pm = context.getPackageManager();
            PackageInfo pi = pm.getPackageInfo(context.getPackageName(), 0);
            return pi.versionName;
        } catch (Exception e) {
            Log.e("VersionInfo", "Exception", e);
        }
        return versionName + "版本名称获取失败";
    }

    /**
     * 解析域名获取IP数组
     *
     * @param host
     *
     * @return
     */
    public static void parseHostGetIPAddress(final String host) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                String[] ipAddressArr = null;
                try {
                    InetAddress[] inetAddressArr = InetAddress.getAllByName(host);
                    if (inetAddressArr != null && inetAddressArr.length > 0) {
                        ipAddressArr = new String[inetAddressArr.length];
                        for (int i = 0; i < inetAddressArr.length; i++) {
                            ipAddressArr[i] = inetAddressArr[i].getHostAddress();
                        }
                    }
                } catch (UnknownHostException e) {
                    e.printStackTrace();
                    onDomainNameTransferIPListener.OnDomainNameTransferIP(ipAddressArr);
                }

                onDomainNameTransferIPListener.OnDomainNameTransferIP(ipAddressArr);
            }
        }).start();
    }

    /**
     * 域名解析成IP
     * @param host
     * @return
     */
    public static String[] parseHostGetIPAddress2(String host) {
        String[] ipAddressArr = null;
        try {
            InetAddress[] inetAddressArr = InetAddress.getAllByName(host);
            if (inetAddressArr != null && inetAddressArr.length > 0) {
                ipAddressArr = new String[inetAddressArr.length];
                for (int i = 0; i < inetAddressArr.length; i++) {
                    ipAddressArr[i] = inetAddressArr[i].getHostAddress();
                }
            }
        } catch (UnknownHostException e) {
            e.printStackTrace();
            return null;
        }
        return ipAddressArr;
    }

    /**
     * 检测域名 IP是否合法
     *
     * @param str
     * @return
     */
    public static boolean isURL2(String str) {
        String regex = "^((https|http|ftp|rtsp|mms)?://)"
                + "?(([0-9a-z_!~*'().&=+$%-]+: )?[0-9a-z_!~*'().&=+$%-]+@)?" //ftp的user@
                + "(([0-9]{1,3}\\.){3}[0-9]{1,3}" // IP形式的URL- 199.194.52.184
                + "|" // 允许IP和DOMAIN（域名）
                + "([0-9a-z_!~*'()-]+\\.)*" // 域名- www.
//                 + "([0-9a-z][0-9a-z-]{0,61})?[0-9a-z]\\." // 二级域名
                + "[a-z]{2,6})" // first level domain- .com or .museum
                + "(:[0-9]{1,4})?" // 端口- :80
                + "((/?)|" // a slash isn't required if there is no file name
                + "(/[0-9a-z_!~*'().;?:@&=+$,%#-]+)+/?)$";
        return match(regex, str);
    }

    /**
     * @param regex 正则表达式字符串
     * @param str   要匹配的字符串
     * @return 如果str 符合 regex的正则表达式格式,返回true, 否则返回 false;
     */
    private static boolean match(String regex, String str) {
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(str);
        return matcher.matches();
    }

    /**
     * 获取APP信息
     *
     * @return
     */
    public static String getDevInformation(Context context) {
        String str = "APP名称：手机蓝牙网关\n" + "APP版本号：" + getAppVersionName(context) + "\n" + "发布日期:" + BuildConfig.BUILD_TIME + "\n";
        if (Build.VERSION.SDK_INT <= 20) {
            return str + "手机版本：4.4";
        } else if (Build.VERSION.SDK_INT == 21) {
            return str + "手机版本：5.0";
        }
        if (Build.VERSION.SDK_INT == 22) {
            return str + "手机版本：5.1";
        }
        if (Build.VERSION.SDK_INT == 23) {
            return str + "手机版本：6.0";
        }
        if (Build.VERSION.SDK_INT == 24) {
            return str + "手机版本：7.0";
        }
        if (Build.VERSION.SDK_INT == 25) {
            return str + "手机版本：7.1";
        }
        if (Build.VERSION.SDK_INT == 26) {
            return str + "手机版本：8.0";
        }
        if (Build.VERSION.SDK_INT == 27) {
            return str + "手机版本：8.1";
        }
        if (Build.VERSION.SDK_INT == 28) {
            return str + "手机版本：9.0";
        }
        if (Build.VERSION.SDK_INT == 29) {
            return str + "手机版本：10.0";
        }
        if (Build.VERSION.SDK_INT == 30) {
            return str + "手机版本：11.0";
        }

        return str;
    }

    public void setOnDomainNameTransferIPListener(OnDomainNameTransferIPListener onDomainNameTransferIPListener) {
        this.onDomainNameTransferIPListener = onDomainNameTransferIPListener;
    }


    public interface OnDomainNameTransferIPListener {
        void OnDomainNameTransferIP(String[] data);

    }


}
